USE [Demokratia]
GO

/****** Object:  Trigger [dbo].[TR_Alegacion_AfterInsertDelete]    Script Date: 30/04/2021 22:46:54 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO



-- =============================================
-- Author:		Josué Martí
-- Create date: 19/04/2021
-- Description:	Disparador que actualiza los indicadores 
--				relacionados con las alegaciones
-- =============================================
CREATE TRIGGER [dbo].[TR_Alegacion_AfterInsertDelete] 
   ON  [dbo].[Alegacion] 
   AFTER INSERT
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE 
    @proximoProceso NVARCHAR(50),
	@integrantes INT,
	@integrantesConAlegaciones INT,
	@valor DECIMAL(18,3),
	@Rsp NVARCHAR(200);

	SELECT TOP 1 @proximoProceso = Nombre
	FROM dbo.ProcesoElectoral
	WHERE HoraInicioVotaciones > GETDATE()
	ORDER BY FechaElecciones ASC

	IF NOT EXISTS (SELECT NumeroAlegacion FROM inserted WHERE NombreProceso=@proximoProceso)
	BEGIN
		RETURN;
	END

	SELECT @integrantes = COUNT(NumeroDniONie)
	FROM dbo.IntegranteMesa
	WHERE NombreProceso = @proximoProceso

	SELECT @integrantesConAlegaciones = COUNT( DISTINCT NumeroDniONie)
	FROM dbo.Alegacion
	WHERE NombreProceso = @proximoProceso
	
	SET @valor = IIF(@integrantes IS NULL OR @integrantes=0, 0, CONVERT(DECIMAL(18,3),@integrantesConAlegaciones * 100) / CONVERT(DECIMAL(18,3),@integrantes))

	EXEC dbo.usp_IndicadorUnico_InsertarOActualizar
		6,
		@proximoProceso,
		@valor,
		@Rsp OUTPUT

END
GO
ALTER TABLE [dbo].[Alegacion] ENABLE TRIGGER [TR_Alegacion_AfterInsertDelete]
GO
/****** Object:  Trigger [dbo].[TR_CensoPorProceso_AfterUpdate]    Script Date: 30/04/2021 22:46:54 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO


-- =============================================
-- Author:		Josué Martí
-- Create date: 25/03/2021
-- Description:	Disparador que actualiza los indicadores 
--				relacionados con el campo votado
-- =============================================
CREATE TRIGGER [dbo].[TR_CensoPorProceso_AfterUpdate] 
   ON  [dbo].[CensoPorProceso] 
   AFTER UPDATE
AS 
BEGIN
	SET NOCOUNT ON;

	IF NOT UPDATE (Votado) 
		RETURN;

	DECLARE 
    @nombreProceso NVARCHAR(50),  
	@idMunicipio INT,
	@nombreMunicipio NVARCHAR(100),  
	@posiblesVotantes INT,
    @votantes INT,
	@nuevoValor DECIMAL(18,3),
	@actualValor DECIMAL(18,3),
	@Rsp NVARCHAR(200);

	-- CALCULAMOS EL INDICADOR 1, PORCENTAJE DE PARTICIPACIÓN DE ESTE PROCESO ELECTORAL

	DECLARE cursor_procesos CURSOR
	FOR SELECT DISTINCT NombreProceso
    FROM inserted I
		
	OPEN cursor_procesos;

	FETCH NEXT FROM cursor_procesos INTO @nombreProceso;

	WHILE @@FETCH_STATUS = 0
		BEGIN
			
			SELECT @posiblesVotantes = COUNT(NumeroDniONie)
			FROM dbo.CensoPorProceso
			WHERE NombreProceso = @nombreProceso

			SELECT @votantes = COUNT(NumeroDniONie)
			FROM dbo.CensoPorProceso
			WHERE NombreProceso = @nombreProceso And Votado=1

			SET @nuevoValor = IIF(@posiblesVotantes IS NULL, 0, CONVERT(DECIMAL(18,3),@votantes * 100) / CONVERT(DECIMAL(18,3),@posiblesVotantes))

			SELECT @actualValor = Valor FROM dbo.IndicadorUnico WHERE TipoIndicador = 1

			IF (@actualValor IS NULL OR @actualValor < @nuevoValor)
			BEGIN
				EXEC dbo.usp_IndicadorUnico_InsertarOActualizar
					1,
					@nombreProceso,
					@nuevoValor,
					@Rsp OUTPUT
			END

			FETCH NEXT FROM cursor_procesos INTO @nombreProceso;
		END;

	CLOSE cursor_procesos;

	DEALLOCATE cursor_procesos;

	-- CALCULAMOS EL INDICADOR 2, PORCENTAJE DE PARTICIPACIÓN POR MUNICIPIO

	DECLARE cursor_municipios CURSOR
	FOR SELECT DISTINCT I.IdMunicipio, D.Nombre
    FROM inserted I
		INNER JOIN dbo.DemarcacionElectoral D ON I.IdMunicipio = D.Id
		
	OPEN cursor_municipios;

	FETCH NEXT FROM cursor_municipios INTO @idMunicipio, @nombreMunicipio;

	WHILE @@FETCH_STATUS = 0
		BEGIN
			
			SELECT @posiblesVotantes = COUNT(NumeroDniONie)
			FROM dbo.CensoPorProceso
			WHERE IdMunicipio = @idMunicipio

			SELECT @votantes = COUNT(NumeroDniONie)
			FROM dbo.CensoPorProceso
			WHERE IdMunicipio = @idMunicipio And Votado=1

			SET @nuevoValor = IIF(@posiblesVotantes IS NULL, 0, CONVERT(DECIMAL(18,3),@votantes * 100) / CONVERT(DECIMAL(18,3),@posiblesVotantes))

			SELECT @actualValor = Valor FROM dbo.IndicadorUnico WHERE TipoIndicador = 2

			IF (@actualValor IS NULL OR @actualValor < @nuevoValor)
			BEGIN
				EXEC dbo.usp_IndicadorUnico_InsertarOActualizar
					2,
					@nombreMunicipio,
					@nuevoValor,
					@Rsp OUTPUT
			END

			FETCH NEXT FROM cursor_municipios INTO @idMunicipio, @nombreMunicipio;
		END;

	CLOSE cursor_municipios;

	DEALLOCATE cursor_municipios;

	-- CALCULAMOS EL INDICADOR 4, INCREMENTO o DECREMENTO DEL TOTAL DE VOTOS DEL ULTIMO
	-- PROCESO RESPECTO AL ANTERIOR

	DECLARE @ultimoProceso NVARCHAR(50)
	DECLARE @fechaUltimoProceso NVARCHAR(50)
	DECLARE @penultimoProceso NVARCHAR(50)
	DECLARE @votantesUltimoProceso INT
	DECLARE @votantesPenultimoProceso INT


	SELECT TOP 1 @ultimoProceso = Nombre, @fechaUltimoProceso = FechaElecciones
	FROM dbo.ProcesoElectoral
	WHERE HoraInicioVotaciones < GETDATE()
	ORDER BY FechaElecciones DESC

	SELECT TOP 1 @penultimoProceso = Nombre
	FROM dbo.ProcesoElectoral
	WHERE HoraInicioVotaciones < @fechaUltimoProceso
	ORDER BY FechaElecciones DESC

	IF NOT EXISTS (SELECT NombreProceso FROM inserted WHERE NombreProceso=@ultimoProceso OR NombreProceso=@penultimoProceso)
	BEGIN
		RETURN;
	END
	
	SELECT @votantesUltimoProceso = COUNT(NumeroDniONie)
	FROM dbo.CensoPorProceso
	WHERE NombreProceso = @ultimoProceso And Votado=1
	
	SELECT @votantesPenultimoProceso = COUNT(NumeroDniONie)
	FROM dbo.CensoPorProceso
	WHERE NombreProceso = @penultimoProceso And Votado=1

	SET @nuevoValor = IIF(@votantesPenultimoProceso IS NULL OR @votantesPenultimoProceso = 0, 0, CONVERT(DECIMAL(18,3),(@votantesUltimoProceso - @votantesPenultimoProceso) * 100) / CONVERT(DECIMAL(18,3),@votantesPenultimoProceso))

	EXEC dbo.usp_IndicadorUnico_InsertarOActualizar
					4,
					@ultimoProceso,
					@nuevoValor,
					@Rsp OUTPUT
END
GO
ALTER TABLE [dbo].[CensoPorProceso] ENABLE TRIGGER [TR_CensoPorProceso_AfterUpdate]
GO
/****** Object:  Trigger [dbo].[TR_IntegranteMesa_AfterDelete]    Script Date: 30/04/2021 22:46:54 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO


-- =============================================
-- Author:		Josué Martí
-- Create date: 28/03/2021
-- Description:	Disparador que actualiza los indicadores 
--				relacionados con los integrantes de mesa
-- =============================================
CREATE TRIGGER [dbo].[TR_IntegranteMesa_AfterDelete] 
   ON  [dbo].[IntegranteMesa] 
   AFTER DELETE
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE 
    @numeroDniONie NVARCHAR(50),
	@veces INT,
	@valor DECIMAL(18,3),
	@Rsp NVARCHAR(200);

	DECLARE cursor_integrantes CURSOR
	FOR SELECT DISTINCT NumeroDniONie
    FROM deleted

	OPEN cursor_integrantes;

	FETCH NEXT FROM cursor_integrantes INTO @numeroDniONie;

	WHILE @@FETCH_STATUS = 0
		BEGIN
			
			SELECT @veces = COUNT(NumeroDniONie)
			FROM dbo.IntegranteMesa
			WHERE NumeroDniONie = @numeroDniONie

			IF @veces = 2 
			BEGIN
				SELECT @valor = Valor FROM dbo.IndicadorUnico WHERE TipoIndicador=5
				SET @valor = @valor - 1

				EXEC dbo.usp_IndicadorUnico_InsertarOActualizar
					5,
					'',
					@valor,
					@Rsp OUTPUT

			END
			
			
			FETCH NEXT FROM cursor_integrantes INTO @numeroDniONie;
		END;

	CLOSE cursor_integrantes;

	DEALLOCATE cursor_integrantes;
    
END
GO
ALTER TABLE [dbo].[IntegranteMesa] ENABLE TRIGGER [TR_IntegranteMesa_AfterDelete]
GO
/****** Object:  Trigger [dbo].[TR_IntegranteMesa_AfterInsert]    Script Date: 30/04/2021 22:46:54 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO


-- =============================================
-- Author:		Josué Martí
-- Create date: 28/03/2021
-- Description:	Disparador que actualiza los indicadores 
--				relacionados con los integrantes de mesa
-- =============================================
CREATE TRIGGER [dbo].[TR_IntegranteMesa_AfterInsert] 
   ON  [dbo].[IntegranteMesa] 
   AFTER INSERT
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE 
    @numeroDniONie NVARCHAR(50),
	@veces INT,
	@valor DECIMAL(18,3),
	@Rsp NVARCHAR(200);

	DECLARE cursor_integrantes CURSOR
	FOR SELECT DISTINCT NumeroDniONie
    FROM inserted I

	OPEN cursor_integrantes;

	FETCH NEXT FROM cursor_integrantes INTO @numeroDniONie;

	WHILE @@FETCH_STATUS = 0
		BEGIN
			
			SELECT @veces = COUNT(NumeroDniONie)
			FROM dbo.IntegranteMesa
			WHERE NumeroDniONie = @numeroDniONie

			IF @veces = 3 
			BEGIN
				SELECT @valor = Valor FROM dbo.IndicadorUnico WHERE TipoIndicador=5
				IF @valor IS NULL
					SET @valor=0
				SET @valor = @valor + 1

				EXEC dbo.usp_IndicadorUnico_InsertarOActualizar
					5,
					'',
					@valor,
					@Rsp OUTPUT

			END
			
			
			FETCH NEXT FROM cursor_integrantes INTO @numeroDniONie;
		END;

	CLOSE cursor_integrantes;

	DEALLOCATE cursor_integrantes;
    
END
GO
ALTER TABLE [dbo].[IntegranteMesa] ENABLE TRIGGER [TR_IntegranteMesa_AfterInsert]
GO
/****** Object:  Trigger [dbo].[TR_Votos_AfterInsertDeleteUpdate]    Script Date: 30/04/2021 22:46:54 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- =============================================
-- Author:		Josué Martí
-- Create date: 25/03/2021
-- Description:	Disparador que actualiza los indicadores 
--				relacionados con los votos
-- =============================================
CREATE TRIGGER [dbo].[TR_Votos_AfterInsertDeleteUpdate] 
   ON  [dbo].[Votos] 
   AFTER INSERT,DELETE,UPDATE
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE 
    @anyo INT,  
	@votosTotales INT,
    @votosTelematicos INT,
	@valor DECIMAL(18,3),
	@Rsp NVARCHAR(200);

	DECLARE cursor_votos CURSOR
	FOR SELECT DISTINCT YEAR(P.FechaElecciones)
    FROM inserted I
		INNER JOIN dbo.ProcesoElectoral P ON I.NombreProceso = P.Nombre

	OPEN cursor_votos;

	FETCH NEXT FROM cursor_votos INTO @anyo;

	WHILE @@FETCH_STATUS = 0
		BEGIN
			
			SELECT @votosTotales = SUM(V.Presenciales + V.Telematicos),
				@votosTelematicos = SUM(V.Telematicos)
			FROM dbo.Votos V
				INNER JOIN dbo.ProcesoElectoral P ON V.NombreProceso = P.Nombre
			WHERE YEAR(P.FechaElecciones) = @anyo

			SET @valor = IIF(@votosTotales IS NULL, 0, CONVERT(DECIMAL(18,3),@votosTelematicos * 100) / CONVERT(DECIMAL(18,3),@votosTotales))

			EXEC dbo.usp_IndicadorAnual_InsertarOActualizar
				3,
				@anyo,
				NULL,
				@valor,
				@Rsp OUTPUT

			FETCH NEXT FROM cursor_votos INTO @anyo;
		END;

	CLOSE cursor_votos;

	DEALLOCATE cursor_votos;
    
END
GO
ALTER TABLE [dbo].[Votos] ENABLE TRIGGER [TR_Votos_AfterInsertDeleteUpdate]
GO